@@ -10,9 +10,11 @@ All rights reserved. This program and the accompanying materials are made
 available under the terms of the BSD which accompanies this distribution, and 
 is available at U{http://www.opensource.org/licenses/bsd-license.php}
 '''
+from gi.repository import Gtk as gtk
+from gi.repository import Gdk as gdk
+from gi.repository.Gio import Settings as GSettings
+from gi.repository import GObject
 
-import gtk
-import gobject
 from base_plugin import Plugin
 from accerciser.tools import *
 from message import MessageManager
@@ -20,11 +22,15 @@ import os
 import sys
 import imp
 from accerciser.i18n import _, N_
-import gconf, gc
+import gc
 from accerciser import ui_manager
 
-GCONF_PLUGINVIEWS = '/apps/accerciser/pluginviews'
-GCONF_LAYOUT_SINGLE = '/apps/accerciser/general/layout_single'
+
+GSCHEMA = 'org.a11y.Accerciser'
+PLUGVIEWS_GSCHEMA = 'org.a11y.Accerciser.pluginviews'
+NEWPLUGVIEWS_GSCHEMA = 'org.a11y.Accerciser.newpluginviews'
+NEWPLUGVIEWS_PATH = '/apps/accerciser/newpluginviews/'
+
 
 class PluginView(gtk.Notebook):
   '''
@@ -35,24 +41,24 @@ class PluginView(gtk.Notebook):
   @cvar TARGET_ROOTWIN: Drag and drop target ID for root window.
   @type TARGET_ROOTWIN: integer
   @ivar NOTEBOOK_GROUP: Group ID for detachable tabs.
-  @type NOTEBOOK_GROUP: integer
+  @type NOTEBOOK_GROUP: string
 
   @ivar view_name: Name of view.
   @type view_name: string
   '''
 
   __gsignals__ = {'plugin_drag_end' : 
-                  (gobject.SIGNAL_RUN_FIRST,
-                   gobject.TYPE_NONE, 
-                   (gobject.TYPE_OBJECT,)),
+                  (GObject.SignalFlags.RUN_FIRST,
+                   None, 
+                   (GObject.TYPE_OBJECT,)),
                   'tab_popup_menu' : 	 
-                  (gobject.SIGNAL_RUN_FIRST, 	 
-                   gobject.TYPE_NONE, 	 
-                   (gobject.TYPE_PYOBJECT, 	 
-                    gobject.TYPE_OBJECT))}
+                  (GObject.SignalFlags.RUN_FIRST, 	 
+                   None, 	 
+                   (GObject.TYPE_PYOBJECT, 	 
+                    GObject.TYPE_OBJECT))}
   TARGET_PLUGINVIEW = 0
   TARGET_ROOTWIN = 1
-  NOTEBOOK_GROUP = 1
+  NOTEBOOK_GROUP = 'NOTEBOOK_GROUP'
 
   def __init__(self, view_name):
     '''
@@ -64,9 +70,9 @@ class PluginView(gtk.Notebook):
     gtk.Notebook.__init__(self)
     self.view_name = view_name
     self.set_scrollable(True)
-    self.set_group_id(self.NOTEBOOK_GROUP)
-    self.connect('drag_end', self._onDragEnd)
-    self.connect('drag_data_get', self._onDragDataGet)
+    self.set_group_name(self.NOTEBOOK_GROUP)
+    self.connect('drag-end', self._onDragEnd)
+    self.connect('drag-data-get', self._onDragDataGet)
     self.connect('key-press-event', self._onKeyPress)
     self.connect('button-press-event', self._onButtonPress)
 
@@ -94,7 +100,7 @@ class PluginView(gtk.Notebook):
     @param event: Event object.
     @type event: gtk.dk.Event
     '''
-    if event.keyval == gtk.keysyms.Menu and \
+    if event.keyval == gdk.KEY_Menu and \
           self.get_property('has-focus'):
       page_num = self.get_current_page()
       child = self.get_nth_page(page_num)
@@ -115,7 +121,8 @@ class PluginView(gtk.Notebook):
     '''
     for child in self.getPlugins():
       tab = self.get_tab_label(child)
-      if tab != None and tab.flags() & gtk.MAPPED:
+      # TODO-JH: Don't really know if this is correct
+      if tab != None and tab.get_state_flags() & tab.get_mapped():
         x, y, w, h = self.getTabAlloc(tab)
         if event_x >= x and \
               event_x <= x + w and \
@@ -134,10 +141,14 @@ class PluginView(gtk.Notebook):
     @return: X, Y, width any height coordinates.
     @rtype: tuple
     '''
-    gdk_window = widget.window
-    origin_x, origin_y = gdk_window.get_origin()
-    x, y, width, height = widget.get_allocation()
-    if widget.flags() & gtk.NO_WINDOW:
+    gdk_window = widget.get_window()
+    origin_z, origin_x, origin_y = gdk_window.get_origin()
+    alloc = widget.get_allocation()
+    x, y, width, height = \
+        alloc.x, alloc.y, alloc.width, alloc.height
+    # TODO-JH: Review this conversion
+    has_window = not(widget.get_has_window())
+    if bool(widget.get_state_flags().value_names) & has_window:
       origin_x += x
       origin_y += y
     return origin_x, origin_y, width, height
@@ -158,7 +169,7 @@ class PluginView(gtk.Notebook):
     @type time: float
     '''
     self.dest_type = info
-    selection_data.set(selection_data.target, 8, '')
+    selection_data.set(selection_data.get_target(), 8, '')
 
   def _onDragEnd(self, widget, drag_context):
     '''
@@ -203,9 +214,9 @@ class PluginView(gtk.Notebook):
       name = tab_label
     elif isinstance(child, Plugin):
       name = getattr(child,'plugin_name_localized', None) or child.plugin_name
-    elif child.name:
-      name = child.name
-    gtk.Notebook.append_page(self, child)
+    elif child.get_name():
+      name = child.get_name()
+    gtk.Notebook.append_page(self, child, None)
     gtk.Notebook.reorder_child(self, child, position)
     gtk.Notebook.set_tab_label(self, child, gtk.Label(name))
 
@@ -262,7 +273,7 @@ class PluginView(gtk.Notebook):
 
 class PluginViewWindow(gtk.Window, Tools):
   '''
-  Standalone window with a plugn view.
+  Standalone window with a plugin view.
 
   @ivar plugin_view: Embedded plugin view.
   @type plugin_view: L{PluginView}
@@ -278,15 +289,15 @@ class PluginViewWindow(gtk.Window, Tools):
     self.plugin_view = PluginView(view_name)
     self.add(self.plugin_view)
 
-    cl = gconf.client_get_default()
-    escaped_view_name = '/%s' % gconf.escape_key(view_name, len(view_name))
-    width = cl.get_int(GCONF_PLUGINVIEWS+escaped_view_name+'/width') or 480
-    height = cl.get_int(GCONF_PLUGINVIEWS+escaped_view_name+'/height') or 480
+    gspath = NEWPLUGVIEWS_PATH + view_name.lower().replace(' ', '-') + '/'
+    gsettings = GSettings(schema=NEWPLUGVIEWS_GSCHEMA, path=gspath)
+    width = gsettings.get_int('width')
+    height = gsettings.get_int('height')
     self.set_default_size(width, height)
     self.connect('key_press_event', self._onKeyPress)
     self.plugin_view.connect_after('page_removed', self._onPluginRemoved)
     self.set_title(view_name)
-    self.set_position(gtk.WIN_POS_MOUSE)
+    self.set_position(gtk.WindowPosition.MOUSE)
     self.show_all()
     self.connect('size-allocate', self._onResize)
 
@@ -301,12 +312,10 @@ class PluginViewWindow(gtk.Window, Tools):
     @type allocation: gtk.gdk.Rectangle
     '''
     view_name = self.plugin_view.view_name
-    key_prefix = '%s/%s' % \
-        (GCONF_PLUGINVIEWS, 
-         gconf.escape_key(view_name, len(view_name)))
-    cl = gconf.client_get_default()
-    cl.set_int(key_prefix+'/width', self.allocation.width)
-    cl.set_int(key_prefix+'/height', self.allocation.height)
+    gspath = NEWPLUGVIEWS_PATH + view_name.lower().replace(' ', '-') + '/'
+    gsettings = GSettings(schema=NEWPLUGVIEWS_GSCHEMA, path=gspath)
+    gsettings.set_int('width', self.get_allocated_width())
+    gsettings.set_int('height', self.get_allocated_height())
 
   def _onPluginRemoved(self, pluginview, page, page_num):
     '''
@@ -332,10 +341,10 @@ class PluginViewWindow(gtk.Window, Tools):
     @param event: Event object
     @type event: gtk.gdk.Event
     '''
-    if event.state & gtk.gdk.MOD1_MASK and \
-          event.keyval in xrange(gtk.gdk.keyval_from_name('0'), 
-                                 gtk.gdk.keyval_from_name('9')):
-      tab_num = event.keyval - gtk.gdk.keyval_from_name('0') or 10
+    if event.state & gdk.ModifierType.MOD1_MASK and \
+          event.keyval in xrange(gdk.keyval_from_name('0'), 
+                                 gdk.keyval_from_name('9')):
+      tab_num = event.keyval - gdk.keyval_from_name('0') or 10
       pages_count = self.plugin_view.get_n_pages()
       if pages_count >= tab_num:
         self.plugin_view.focusTab(tab_num - 1)
@@ -353,8 +362,8 @@ class ViewManager(object):
     @type perm_views: list of {PluginView}
     '''
     self._perm_views = perm_views
-    cl = gconf.client_get_default()
-    single = cl.get_bool(GCONF_LAYOUT_SINGLE)
+    gsettings = GSettings(schema=PLUGVIEWS_GSCHEMA)
+    single = gsettings.get_boolean('layout-single')
     self._initViewModel(single)
     self._setupActions()
     
@@ -374,10 +383,10 @@ class ViewManager(object):
       action_name = action.get_name()
       ui_manager.uimanager.add_ui(merge_id, ui_manager.PLUGIN_LAYOUT_PATH, 
                                   action_name, action_name, 
-                                  gtk.UI_MANAGER_MENUITEM, True)
+                                  gtk.UIManagerItemType.MENUITEM, True)
 
 
-  def _onSingleViewToggled(self, action):
+  def _onSingleViewToggled(self, action, data=None):
     '''
     Callback for single view toggle action.
     
@@ -395,14 +404,14 @@ class ViewManager(object):
     '''
     if isinstance(self._view_model, SingleViewModel) == single:
       return
-    cl = gconf.client_get_default()
-    cl.set_bool(GCONF_LAYOUT_SINGLE, single)
+    gsettings = GSettings(schema=PLUGVIEWS_GSCHEMA)
+    gsettings.set_boolean('layout-single', single)
     plugins = self._view_model.getViewedPlugins()
     self._view_model.close()
     del self._view_model
     for plugin in plugins:
-      if plugin.parent:
-        plugin.parent.remove(plugin)
+      if plugin.get_parent():
+        plugin.get_parent().remove(plugin)
     self._initViewModel(single)
     for plugin in plugins:
       self._view_model.addElement(plugin)
@@ -512,7 +521,7 @@ class BaseViewModel(Tools):
     '''
     Add an element to a plugin view. If the element is a message tab, put it as
     the first tab in the main view. If the element is a plugin, check if it's 
-    placement is cached in this instance or read it's position from gconf.
+    placement is cached in this instance or read it's position from gsettings.
     By default a plugin is appended to the main view.
     
     @param element: The element to be added to a view.
@@ -530,7 +539,7 @@ class BaseViewModel(Tools):
   def addPlugin(self, plugin):
     '''
     Add a plugin to the view. Check if it's placement is cached in this 
-    instance or read it's position from gconf. By default a plugin is 
+    instance or read it's position from gsettings. By default a plugin is 
     appended to the main view.
     
     @param plugin: Plugin to add.
@@ -663,13 +672,13 @@ class MultiViewModel(list, BaseViewModel):
   @ivar main_view: Main view.
   @type main_view: L{PluginView}
   @ivar _ignore_insertion: A list of tuples with view and plugin names that
-  should be ignored and not go in to gconf. This is to avoid recursive gconf
-  modification.
+  should be ignored and not go in to gsettings. This is to avoid recursive
+  gsettings modification.
   @type _ignore_insertion: list of tuples
   @ivar _placement_cache: A cache of recently disabled plugins with their 
   placement. allowsthem to be enabled in to the same position.
   @type _placement_cache: dictionary
-  @ivar _closed: Indicator to stop writing plugin remove events to gconf.
+  @ivar _closed: Indicator to stop writing plugin remove events to gsettings.
   @type _closed: boolean
   '''
   COL_NAME = 0
@@ -691,13 +700,13 @@ class MultiViewModel(list, BaseViewModel):
 
   def close(self):
     '''
-    Stops gconf maniputaion.
+    Stops gsettings maniputaion.
     '''
     self._closed = True
 
   def getViewNameForPlugin(self, plugin_name):
     '''
-    Get the view name for a given plugin name as defined in gconf. 
+    Get the view name for a given plugin name as defined in gsettings. 
     Or return name of main view.
     
     @param plugin_name: Plugin's name to lookup view for.
@@ -706,7 +715,7 @@ class MultiViewModel(list, BaseViewModel):
     @return: View name for plugin.
     @rtype: string
     '''
-    plugin_layouts = self._StoredViewsLayout()
+    plugin_layouts = self._getPluginLayouts()
     for view_name in plugin_layouts:
       if plugin_name in plugin_layouts[view_name]:
         return view_name
@@ -821,14 +830,14 @@ class MultiViewModel(list, BaseViewModel):
     '''
     menu = self.Menu(plugin, view.get_toplevel())
     if hasattr(event, 'button'):
-      menu.popup(None, None, None, event.button, event.time)
+      menu.popup(None, None, None, None, event.button, event.time)
     else:
       tab = view.get_tab_label(plugin)
       x, y, w, h = view.getTabAlloc(tab)
-      rect = gtk.gdk.Rectangle(x, y, w, h)
+      rect = gdk.Rectangle(x, y, w, h)
       menu.popup(None, None, 
                  lambda m, r: (r.x+r.width/2, r.y+r.height/2, True), 
-                 0, event.time, rect)
+                 rect, 0, event.time)
   
   def _connectSignals(self, view):
     '''
@@ -837,17 +846,17 @@ class MultiViewModel(list, BaseViewModel):
     @param view: Plugin view to connect.
     @type view: :{PluginView}
     '''
-    if isinstance(view.parent, PluginViewWindow):
-      view.parent.connect('delete_event', Proxy(self._onViewDelete))
-    view.connect('plugin_drag_end', Proxy(self._onPluginDragEnd))
-    view.connect('tab_popup_menu', Proxy(self._onTabPopupMenu))
-    view.connect('page_added', Proxy(self._onViewLayoutChanged), 'added')
-    view.connect('page_removed', Proxy(self._onViewLayoutChanged), 'removed')
-    view.connect('page_reordered', Proxy(self._onViewLayoutChanged), 'reordered')
+    if isinstance(view.get_parent(), PluginViewWindow):
+      view.get_parent().connect('delete-event', Proxy(self._onViewDelete))
+    view.connect('plugin-drag-end', Proxy(self._onPluginDragEnd))
+    view.connect('tab-popup-menu', Proxy(self._onTabPopupMenu))
+    view.connect('page-added', Proxy(self._onViewLayoutChanged), 'added')
+    view.connect('page-removed', Proxy(self._onViewLayoutChanged), 'removed')
+    view.connect('page-reordered', Proxy(self._onViewLayoutChanged), 'reordered')
 
   def _onViewLayoutChanged(self, view, plugin, page_num, action):
     '''
-    Callback for all layout changes. Updates gconf.
+    Callback for all layout changes. Updates gsettings.
     
     @param view: View that emitted the signal.
     @type view: L{PluginView}
@@ -857,14 +866,15 @@ class MultiViewModel(list, BaseViewModel):
     @type page_num: integer
     @param action: Action that triggered this event.
     @type action: string    
-    '''
+    ''' 
     if self._closed or not isinstance(plugin, Plugin): return
     if (view.view_name, plugin.plugin_name) in self._ignore_insertion:
       self._ignore_insertion.remove((view.view_name, plugin.plugin_name))
       return
     if self._placement_cache.has_key(plugin.plugin_name):
       self._placement_cache.pop(plugin.plugin_name)
-    plugin_layouts = self._StoredViewsLayout()
+
+    plugin_layouts = self._getPluginLayouts()
     try:
       plugin_layout = plugin_layouts[view.view_name]
     except KeyError:
@@ -879,10 +889,44 @@ class MultiViewModel(list, BaseViewModel):
     if len(plugin_layout) == 0:
       self._removeView(view)
 
+    self._setPluginLayouts(plugin_layouts)
+
+  def _setPluginLayouts(self, plugin_layouts):
+    self.plugviews = GSettings(schema=PLUGVIEWS_GSCHEMA)
+    self.plugviews.set_strv('top-panel-layout', plugin_layouts.pop('Top panel'))
+    self.plugviews.set_strv('bottom-panel-layout', plugin_layouts.pop('Bottom panel'))
+
+    for plugview in plugin_layouts.keys():
+      gspath = NEWPLUGVIEWS_PATH + plugview.lower().replace(' ','-') + '/'
+      newview = GSettings(schema=NEWPLUGVIEWS_GSCHEMA, path=gspath)
+      newview.set_strv('layout', plugin_layouts[plugview])
+      l = self.plugviews.get_strv('available-newviews')
+      l.append(plugview)
+      self.plugviews.set_strv('available-newviews', l)
+
+  def _getPluginLayouts(self):
+    plugin_layouts= {}
+    self.plugviews = GSettings(schema=PLUGVIEWS_GSCHEMA)
+    plugin_layouts['Top panel'] = self.plugviews.get_strv('top-panel-layout')
+    plugin_layouts['Bottom panel'] = self.plugviews.get_strv('bottom-panel-layout')
+
+    for plugview in self.plugviews.get_strv('available-newviews'):
+      gspath = NEWPLUGVIEWS_PATH + plugview.lower().replace(' ','-') + '/'
+      newview = GSettings(schema=NEWPLUGVIEWS_GSCHEMA, path=gspath)
+      layout = newview.get_strv('layout')
+      if layout:
+        plugin_layouts[plugview] = layout
+      else:
+        l = self.plugviews.get_strv('available-newviews')
+        l.remove(plugview)
+        self.plugviews.set_strv('available-newviews', l)
+    return plugin_layouts
+
+
   def addPlugin(self, plugin):
     '''
     Add a plugin to the view. Check if it's placement is cached in this 
-    instance or read it's position from gconf. By default a plugin is 
+    instance or read it's position from gsettings. By default a plugin is 
     appended to the main view.
     
     @param plugin: Plugin to add.
@@ -894,7 +938,7 @@ class MultiViewModel(list, BaseViewModel):
     else:
       view_name = self.getViewNameForPlugin(plugin.plugin_name)
       view = self._getViewOrNewView(view_name)
-      plugin_layouts = self._StoredViewsLayout()
+      plugin_layouts = self._getPluginLayouts()
       try:
         plugin_layout = plugin_layouts[view.view_name]
       except KeyError:
@@ -910,6 +954,7 @@ class MultiViewModel(list, BaseViewModel):
             index = child_index
             break
       self._ignore_insertion.append((view.view_name, plugin.plugin_name))
+      self._setPluginLayouts(plugin_layouts)
 
     view.insert_page(plugin, position=index)
     view.set_tab_detachable(plugin, True)
@@ -953,7 +998,7 @@ class MultiViewModel(list, BaseViewModel):
     @type new_view_name: string
     '''
     if not plugin or not isinstance(plugin, gtk.Widget): return
-    old_view = plugin.parent
+    old_view = plugin.get_parent()
     new_view = self._getViewOrNewView(new_view_name)
     if old_view is not new_view:
       old_view.remove(plugin)
@@ -1018,16 +1063,17 @@ class MultiViewModel(list, BaseViewModel):
       '''
       menu_item = None
       for view in self.view_manager:
-        menu_item = gtk.RadioMenuItem(menu_item, view.view_name)
+        menu_item = gtk.RadioMenuItem(label = view.view_name)
+        menu_item.set_name(view.view_name)
         menu_item.connect('toggled', self._onItemToggled, view, context_plugin)
-        menu_item.set_active(view == context_plugin.parent)
+        menu_item.set_active(view == context_plugin.get_parent())
         self.append(menu_item)
         menu_item.show()
       menu_item = gtk.SeparatorMenuItem()
       self.append(menu_item)
       menu_item.show()
-      menu_item = gtk.MenuItem(_('<i>_New view...</i>'))
-      menu_item.child.set_use_markup(True)
+      menu_item = gtk.MenuItem(label=_('<i>_New view...</i>'))
+      menu_item.get_child().set_use_markup(True)
       menu_item.connect('activate', self._onItemActivated, 
                         context_plugin, transient_window)
       self.append(menu_item)
@@ -1063,7 +1109,7 @@ class MultiViewModel(list, BaseViewModel):
           self._NewViewDialog(self.view_manager, transient_window)
       response_id = new_view_dialog.run()
       plugin_name = new_view_dialog.getEntryText()
-      if response_id == gtk.RESPONSE_OK and plugin_name:
+      if response_id == gtk.ResponseType.OK and plugin_name:
         self.view_manager.changeView(context_plugin, plugin_name)
       new_view_dialog.destroy()
 
@@ -1083,9 +1129,9 @@ class MultiViewModel(list, BaseViewModel):
         '''
         self.view_manager = view_manager
         gtk.Dialog.__init__(self, _('New View...'), transient_window)
-        self.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK,
-                         gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
-        self.set_default_response(gtk.RESPONSE_OK)
+        self.add_buttons(gtk.STOCK_OK, gtk.ResponseType.OK,
+                         gtk.STOCK_CLOSE, gtk.ResponseType.CLOSE)
+        self.set_default_response(gtk.ResponseType.OK)
         completion = gtk.EntryCompletion()
         complete_model = gtk.ListStore(str)
         for view in self.view_manager:
@@ -1095,7 +1141,8 @@ class MultiViewModel(list, BaseViewModel):
         self.entry = gtk.Entry()
         self.entry.set_completion(completion)
         self.entry.connect('activate', self._onEntryActivate)
-        self.vbox.add(self.entry)
+        self.box = self.get_children()[0]
+        self.box.add(self.entry)
         self.entry.show()
 
       def getEntryText(self):
@@ -1114,30 +1161,5 @@ class MultiViewModel(list, BaseViewModel):
         @param entry: Entry box that was activated.
         @type entry: gtk.Entry
         '''
-        self.response(gtk.RESPONSE_OK)
+        self.response(gtk.ResponseType.OK)
 
-  class _StoredViewsLayout(object):
-    '''
-    Convenience class for emulating a dictionary of all plugin 
-    view layout lists.
-    '''
-    def __init__(self):
-      self.gconf_client = gconf.client_get_default()
-    def __len__(self):
-      view_dirs = self.gconf_client.all_dirs(GCONF_PLUGINVIEWS)
-      return len(view_dirs)
-    def __getitem__(self, key):
-      view_dirs = self.gconf_client.all_dirs(GCONF_PLUGINVIEWS)
-      for dir in view_dirs:
-        if key == gconf.unescape_key(dir.split('/')[-1], 
-                                     len(dir.split('/')[-1])):
-          return GConfListWrapper(dir+'/layout')
-      raise KeyError, key
-    def __setitem__(self, key, value):
-      gconf_key = '%s/%s/layout' % \
-          (GCONF_PLUGINVIEWS, gconf.escape_key(key, len(key)))
-      self.gconf_client.set_list(gconf_key, gconf.VALUE_STRING, value)
-    def __iter__(self):
-      view_dirs = self.gconf_client.all_dirs(GCONF_PLUGINVIEWS)
-      for dir in view_dirs:
-        yield gconf.unescape_key(dir.split('/')[-1], len(dir.split('/')[-1]))
